package com.huiju.welcome.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaPerson;
import com.huiju.welcome.model.TaVisitingLog;
import com.huiju.welcome.service.ITaPersonService;
import com.huiju.welcome.service.ITaVisitingLogService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@RestController
@RequestMapping("/")
public class TaVisitingLogController extends BaseController {
    @Autowired
    private ITaVisitingLogService taVisitingLogService;

    @Autowired
    private ITaPersonService taPersonService;

    @ApiOperation(value = "编辑访问记录", notes = "编辑访问记录")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "TaVisitingLog", name = "taVisitingLog", value = "访问记录")
    })
    @RequestMapping(value = "/visiting_log/{logId}", method = RequestMethod.PUT)
    public ResponseBean update(@RequestBody TaVisitingLog taVisitingLog, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        Integer sales = taVisitingLog.getSalesId();
        if (null != sales && sales.intValue() > 0) {
            // 更新人员主表
            TaPerson taPerson = new TaPerson();
            taPerson.setPersonId(taVisitingLog.getPersonId());
            taPerson.setLastSales(sales);
            taPerson.setLastSalesName(taVisitingLog.getSalesName());
            boolean success = taPersonService.updateSales(taPerson);
            if (!success) {
                responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "更新内容失败, 请重试");
                return responseBean;
            }
        }

        boolean success = taVisitingLogService.updateById(taVisitingLog);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "更新内容失败, 请重试");
            return responseBean;
        }

        responseBean.addSuccess(taVisitingLog);
        return responseBean;
    }

    @ApiOperation(value = "获取访问列表", notes = "获取访问列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageNum", value = "当前页"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageSize", value = "每页数量"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "personId", value = "人员ID"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "beginDate", value = "起始日期"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "endDate", value = "结束日期"),
    })
    @RequestMapping(value = "/visitinglog", method = RequestMethod.GET)
    public ResponseBean list(
            @RequestParam Integer pageNum,
            @RequestParam  Integer pageSize,
            Integer personId,
            String beginDate,
            String endDate,
            HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        if (null != personId && personId < 1) personId = null;
        if (null != beginDate && "".equals(beginDate)) beginDate = null;
        if (null != endDate && "".equals(endDate)) endDate = null;

        Page<TaVisitingLog> page = new Page<>(pageNum, pageSize);
        responseBean.addSuccess(taVisitingLogService.getVisitingList(page, beginDate, endDate, personId));

        return responseBean;
    }

    @ApiOperation(value = "日访问量统计", notes = "日访问量统计")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "startDate", value = "起始日期"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "endDate", value = "结束日期"),
    })
    @RequestMapping(value = "/stats/daily/visitinglog", method = RequestMethod.GET)
    public ResponseBean visitingDailyAmout(@RequestParam String startDate, @RequestParam String endDate, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        DateTimeFormatter df = DateTimeFormatter.ISO_LOCAL_DATE_TIME;
        LocalDateTime startDt = LocalDateTime.parse(startDate, df);
        LocalDateTime endDt = LocalDateTime.parse(endDate, df);

        responseBean.addSuccess(taVisitingLogService.getVisitingDailyAmount(startDt, endDt));
        return responseBean;
    }

    @GetMapping("/miniapp/visiting/timeline")
    public ResponseBean timeline() {
        return taVisitingLogService.getTimeline();
    }
}
