package com.huiju.welcome.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaFirstUsherRecord;
import com.huiju.welcome.model.TaVisitorAppointment;
import com.huiju.welcome.service.ITaVisitorAppointmentService;
import com.huiju.welcome.utils.DateUtils;
import com.huiju.welcome.utils.JWTUtils;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.apache.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.Date;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@RestController
@RequestMapping("/")
public class TaVisitorAppointmentController extends BaseController {
    @Autowired
    ITaVisitorAppointmentService taVisitorAppointmentService;

    @GetMapping("/appointment")
    public ResponseBean list(
            @RequestParam(defaultValue = "1") Integer page,
            @RequestParam(defaultValue = "10") Integer size,
            String name,
            String phone,
            String platNumber
    ) {
        IPage<TaVisitorAppointment> pg = new Page<>(page, size);
        QueryWrapper<TaVisitorAppointment> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(name), "person_name", name);
        queryWrapper.like(!StringUtils.isEmpty(phone), "phone", phone);
        queryWrapper.like(!StringUtils.isEmpty(platNumber), "plat_number", platNumber);
        queryWrapper.orderByDesc("create_date");

        IPage<TaVisitorAppointment> result = taVisitorAppointmentService.page(pg, queryWrapper);

        ResponseBean resp = new ResponseBean();
        resp.addSuccess(result);
        return resp;
    }

    @GetMapping("/appointment/{id}")
    public ResponseBean getByID(@PathVariable Integer id) {
        TaVisitorAppointment taVisitorAppointment = taVisitorAppointmentService.getById(id);
        return ResponseBean.success(taVisitorAppointment);
    }

    @PostMapping("/miniapp/appointment")
    public ResponseBean save(@RequestBody TaVisitorAppointment taVisitorAppointment, HttpServletRequest request) {
        taVisitorAppointment.setStatus(StatusUtils.Normal);
        taVisitorAppointment.setCreateDate(LocalDateTime.now());

        // 是否重复预约
        ResponseBean resp = taVisitorAppointmentService.check(taVisitorAppointment);
        if (resp.getCode() != HttpStatus.SC_OK) {
            return resp;
        }

        taVisitorAppointment.setCreateUser(Integer.valueOf(JWTUtils.getSubject(request)));

        boolean success = taVisitorAppointmentService.save(taVisitorAppointment);
        if (success) {
            return ResponseBean.success(taVisitorAppointment);
        } else {
            return ResponseBean.error("保存数据失败", HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }
    }

    @PutMapping("/appointment/{id}")
    public ResponseBean update(@PathVariable Integer id, @RequestBody String paramStr) {
        if (StringUtils.isEmpty(paramStr)) {
            return ResponseBean.error("请设置保存内容", HttpStatus.SC_BAD_REQUEST);
        }

        JSONObject params = JSONObject.parseObject(paramStr);
        UpdateWrapper<TaVisitorAppointment> wrapper = new UpdateWrapper<>();
        wrapper.eq("appointment_id", id);
        wrapper.set("person_id", params.getInteger("personId"));
        wrapper.set("person_name", params.getString("personName"));
        wrapper.set("phone", params.getString("phone"));
        wrapper.set("visit_date", params.getString("visitDate"));
        wrapper.set("person_num", params.getInteger("personNum"));
        wrapper.set("car_num", params.getInteger("carNum"));
        wrapper.set("car_model", params.getString("carModel"));
        wrapper.set("plat_number", params.getString("platNumber"));
        wrapper.set("status", params.getInteger("status"));

        Date dt = params.getDate("visitDate");
        if (null != dt) {
            wrapper.set("visit_date", DateUtils.date2LocalDateTime(dt));
        }

        boolean success = taVisitorAppointmentService.update(new TaVisitorAppointment(), wrapper);
        if (success) {
            return ResponseBean.success(null);
        } else {
            return ResponseBean.error("更新数据失败", HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }
    }
    @ApiOperation(value = "访客预约信息",notes = "访客预约信息")
    @RequestMapping(value="/visitorAppointmentList",method= RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "platNumber", value = "车牌"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "personName", value = "预约人姓名"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "phone", value = "预约人电话")})
    public ResponseBean visitorAppointmentList(@RequestParam(defaultValue = "1") int pageNum,
                                               @RequestParam(defaultValue = "10") int pageSize,
                                               @RequestParam(value = "platNumber",required = false) String platNumber,
                                               @RequestParam(value = "personName",required = false) String personName,
                                               @RequestParam(value = "phone",required = false) String phone){
        IPage<TaVisitorAppointment> pg = new Page<>(pageNum, pageSize);
        ResponseBean result = taVisitorAppointmentService.visitorAppointmentList(pg,platNumber,personName,phone);
        return result;
    }

}
