package com.huiju.welcome.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.utils.JWTUtils;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.JwtException;
import io.jsonwebtoken.Jwts;
import org.apache.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.HashMap;
import java.util.Map;

/**
 * 登录校验拦截器
 * @ClassName: AccessInterceptor
 * @Description:
 * @author weiyb
 * @date 2018年5月7日 上午11:51:53
 */
@Component
public class AccessInterceptor implements HandlerInterceptor {
    // 正则白名单
    private String[] whiteList = {
        "/login",
        "/ping",
        "/device",
        "/familiar",
        "/stranger",
        "/screen",
        "/camera",
        "/websocket",
        "/miniapp/login"
    };

    /*
     * 视图渲染之后的操作
     */
    @Override
    public void afterCompletion(HttpServletRequest arg0, HttpServletResponse arg1, Object arg2, Exception arg3)
            throws Exception {

    }

    /*
     * 处理请求完成后视图渲染之前的处理操作
     */
    @Override
    public void postHandle(HttpServletRequest arg0, HttpServletResponse arg1, Object arg2, ModelAndView arg3)
            throws Exception {

    }

    /*
     * 进入controller层之前拦截请求
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {
        // 白名单跳过
        if (inWhiteList(request)) {
            return true;
        }

        return isWechat(request) ? checkJWT(request, response, obj) : checkSession(request, response, obj);
    }

    private boolean checkJWT(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {

//        // 消息头必须包含 authorization 字段
//        String jwtHeader = request.getHeader("authorization");
//        if (null == jwtHeader || "".equals(jwtHeader)) {
//            ResponseBean resp = ResponseBean.error("请先进行系统登录操作", HttpStatus.SC_UNAUTHORIZED);
//
//            response.addHeader("Content-type", "application/json");
//            response.getOutputStream().write(JSONObject.toJSONBytes(resp));
//            return false;
//        }
//
//        String[] jwtAuths = jwtHeader.split("Bearer ");
//        if (null == jwtAuths || jwtAuths.length < 2) {
//            ResponseBean resp = ResponseBean.error("权限验证非法, 数据格式不正确", HttpStatus.SC_UNAUTHORIZED);
//
//            response.addHeader("Content-type", "application/json");
//            response.getOutputStream().write(JSONObject.toJSONBytes(resp));
//            return false;
//        }
//
//        try {
//            // 只尝试去解析
//            Jwts.parser().setSigningKey(JWTUtils.key).parseClaimsJws(jwtAuths[1]);
//        } catch (ExpiredJwtException e) {
//            ResponseBean resp = ResponseBean.error("权限验证非法, Token 已过期", HttpStatus.SC_UNAUTHORIZED);
//            response.addHeader("Content-type", "application/json");
//            response.getOutputStream().write(JSONObject.toJSONBytes(resp));
//            return false;
//
//        } catch (JwtException e) {
//            ResponseBean resp = ResponseBean.error("权限验证非法, " + e.getMessage(), HttpStatus.SC_UNAUTHORIZED);
//
//
//            response.addHeader("Content-type", "application/json");
//            response.getOutputStream().write(JSONObject.toJSONBytes(resp));
//            return false;
//        }

        return true;
    }

    private boolean checkSession(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {

        HttpSession httpSession = request.getSession();
        String token = (String) httpSession.getAttribute("token");

        // 如果未登录
        if (null == token || "".equals(token)) {
            Map error = new HashMap<String, Object>();
            error.put("code", HttpStatus.SC_UNAUTHORIZED);
            error.put("message", "请先登录系统");
            response.addHeader("Content-type", "application/json");
            response.getOutputStream().write(JSONObject.toJSONBytes(error));
            return false;
        }
        return true;
    }

    private boolean isWechat(HttpServletRequest request) {
        String ua = request.getHeader("user-agent");
        return ua.contains("micromessenger") || ua.contains("MicroMessenger");
    }

    private boolean inWhiteList(HttpServletRequest request) {
        String requestURI = request.getRequestURI();

        for (String it : whiteList) {
            if (requestURI.contains(it)) {
                return true;
            }
        }

        return false;
    }

}
